uniform vec2 mouse;
uniform vec2 resolution;
uniform sampler2D backbuffer;
uniform int frame;
uniform float speed;


out vec4 col;


bvec3 vec3GreaterThan(in vec3 a, in vec3 b) {
    return bvec3(a.x > b.x, a.y > b.y, a.z > b.z);
}


bvec3 bvec3And(in bvec3 a, in bvec3 b) {
	return bvec3(a.x && b.x, a.y && b.y, a.z && b.z);
}


bvec3 vec3Equal(in vec3 a, in vec3 b) {
	return bvec3(a.x == b.x, a.y == b.y, a.z == b.z);
}


bvec3 bvec3Not(in bvec3 a) {
	return bvec3(!a.x, !a.y, !a.z);
}


bvec3 bvec3Or(in bvec3 a, in bvec3 b) {
	return bvec3(a.x || b.x, a.y || b.y, a.z || b.z);
}


void main()
{
    vec2 uv = gl_FragCoord.xy/resolution.xy;
    vec2 cursor = vec2(mouse.x, 1. - mouse.y) * resolution; // adjusted cursor position, bottom left is 0,0
	
	if (frame < 2) {
		col = vec4(0, 0, 0, 1);
	}
	
	else {if (fract(float(frame - 1) * speed) >= fract(float(frame) * speed)) {
		
		ivec3 living_neighbours = ivec3(0);
		bvec3 this_living = vec3GreaterThan(texelFetch(backbuffer, ivec2(gl_FragCoord.xy), 0).xyz, vec3(0.5));
		
		for (int x_off = -1; x_off <= 1; x_off++) {
			for (int y_off = -1; y_off <= 1; y_off++) {
				if (!(x_off == 0 && y_off == 0)) {
					vec4 current_fetch = texelFetch(backbuffer, ivec2(mod(gl_FragCoord.xy + vec2(x_off, y_off), resolution + vec2(1))), 0);
					
					living_neighbours += ivec3(vec3GreaterThan(current_fetch.xyz, vec3(.5)));
				}
			}
		}
		bvec3 survives = bvec3And(bvec3And(vec3GreaterThan(vec3(living_neighbours), vec3(1)), vec3GreaterThan(vec3(4), vec3(living_neighbours))), this_living);
		bvec3 becomes_alive = bvec3And(vec3Equal(vec3(living_neighbours), vec3(3)), bvec3Not(this_living));
			
		bvec3 states = bvec3Or(survives, becomes_alive);
			
		float dist = length(cursor - gl_FragCoord.xy);
		if (dist < 17.) {
			states = bvec3Or(states, bvec3(dist < 15., dist < 16., dist < 17.));
		}
		
		col = vec4(states, 1.);
		}
	
		else {
		col = texelFetch(backbuffer, ivec2(gl_FragCoord.xy), 0);
		if (col.w <= 0.) col = vec4(0., 0., 0., 1.);
		}
	}
}